local Recount2 = Recount2
local Recount2_Widgets = Recount2.Widgets

local tinsert = table.insert
local tremove = table.remove

local dropdownRowHeight = 12
local dropdownRowSpacing = 2


local function SetFontSize(string, size)
	local Font, Height, Flags = string:GetFont()
	string:SetFont(Font, size, Flags)
end

local curLevel = 0
local checkLevel = 0
local dropFrame
local dropFrames = {}
local buttonRows = {}
local inUse
local position = dropdownRowSpacing

local function FreeItem(item)
	item:Hide()
	tinsert(item.storage, item)
end

local function DropdownUpdate(self)
	if checkLevel==self.level and not MouseIsOver(self, 4, -4, -4, 4) then
		self:Hide()

		for k, v in ipairs(inUse) do
			FreeItem(v)
			inUse[k] = nil
		end

		curLevel = curLevel-1

		if curLevel>0 then
			dropFrame = dropFrames[curLevel]
			inUse = dropFrame.inUse
		end
	end
end

local function IncrementDropdownLevel()
	curLevel = curLevel + 1

	if curLevel>#dropFrames then
		dropFrame = CreateFrame("FRAME", nil, UIParent)
		dropFrame.background = dropFrame:CreateTexture(nil, "BACKGROUND")
		dropFrame.background:SetTexture(0, 0, 0, 0.75)
		dropFrame.background:ClearAllPoints()
		dropFrame.background:SetAllPoints(dropFrame)
		dropFrame.background:Show()
		dropFrame:SetWidth(100)
		dropFrame.OnUpdate = DropdownUpdate
		dropFrame:SetScript("OnUpdate", dropFrame.OnUpdate)
		dropFrame:EnableMouse()

		dropFrame.level = curLevel
		dropFrame.inUse = Recount2:GetTable()
		dropFrame.textRows = {}
		dropFrame.dividerRows = {}

		Recount2:AddWindow(dropFrame)
		dropFrames[curLevel] = dropFrame
	else
		dropFrame = dropFrames[curLevel]
	end

	inUse = dropFrame.inUse
	position = dropdownRowSpacing
end

function Recount2_Widgets:DropdownClear()
	for i = 1, curLevel do
		dropFrame = dropFrames[i]
		dropFrame:Hide()
		dropFrame:SetWidth(100)

		local inUse = dropFrame.inUse
		for k, v in ipairs(inUse) do
			FreeItem(v)
			inUse[k] = nil
		end

		if dropFrame:HasScript("OnUpdate") then
			dropFrame:SetScript("OnUpdate", nil)
		end
	end

	curLevel = 0
	IncrementDropdownLevel()
end

local function PositionTooltip()
	local x, y = GetCursorPosition()
	local scale = dropFrame:GetEffectiveScale() 
	dropFrame:ClearAllPoints()
	dropFrame:SetPoint("BOTTOMLEFT", UIParent, "BOTTOMLEFT", (x+1)/scale, (y+1)/scale)

	Recount2:SetWindowTop(dropFrame)
end

function Recount2_Widgets:DropdownOpen(bindToMouse)
	local x, y = GetCursorPosition()
	local scale = dropFrame:GetEffectiveScale() 
	dropFrame:ClearAllPoints()
	
	
	Recount2:SetWindowTop(dropFrame)
	dropFrame:Show()

	dropFrame.bindToMouse = bindToMouse

	if bindToMouse then
		dropFrame:SetScript("OnUpdate", PositionTooltip)
		dropFrame:EnableMouse(false)
		dropFrame:SetPoint("BOTTOMLEFT", UIParent, "BOTTOMLEFT", (x+1)/scale, (y+1)/scale)
	else
		dropFrame:SetScript("OnUpdate", dropFrame.OnUpdate)
		dropFrame:EnableMouse(true)
		dropFrame:SetPoint("TOPRIGHT", UIParent, "BOTTOMLEFT", (x+4)/scale, (y+4)/scale)
	end

	checkLevel = curLevel

	--Now increment the level
	IncrementDropdownLevel()
end



function Recount2_Widgets:DropdownAddText(text)
	local textRow
	if #dropFrame.textRows>0 then
		textRow = tremove(dropFrame.textRows)
		textRow:Show()
	else
		textRow = dropFrame:CreateFontString(nil ,"OVERLAY","GameFontHighlightSmall")
		Recount2:AddFontString(textRow)
		SetFontSize(textRow, dropdownRowHeight)
		textRow:SetJustifyH("LEFT")
		textRow.storage = dropFrame.textRows
	end

	textRow:SetText(text)
	local w = textRow:GetStringWidth()
	if (w+4)>dropFrame:GetWidth() then
		dropFrame:SetWidth(w+4)
	end
	textRow:SetPoint("TOPLEFT", dropFrame, "TOPLEFT", 2, -position)
	position = position + dropdownRowHeight + dropdownRowSpacing
	dropFrame:SetHeight(position)

	tinsert(inUse, textRow)
end

function Recount2_Widgets:DropdownAddDivider()
	local dividerRow
	if #dropFrame.dividerRows>0 then
		dividerRow = tremove(dropFrame.dividerRows)
		dividerRow:Show()
	else
		dividerRow = dropFrame:CreateTexture(nil ,"BORDER")
		dividerRow:SetTexture(1, 1, 1, 0.5)
		dividerRow:SetHeight(1)
		dividerRow.storage = dropFrame.dividerRows
	end

	dividerRow:SetPoint("TOPLEFT", dropFrame, "TOPLEFT", 2, -position)
	dividerRow:SetPoint("TOPRIGHT", dropFrame, "TOPRIGHT", -2, -position)
	position = position + 1 + dropdownRowSpacing
	dropFrame:SetHeight(position)

	tinsert(inUse, dividerRow)
end

local function ButtonOnEnter(self)
	self.Highlight:Show()
end

local function ButtonOnLeave(self)
	self.Highlight:Hide()
end

local function ButtonOnClick(self)
	dropFrame:Hide()

	if self.func then
		self.func(unpack(self.args))
	end
end

function Recount2_Widgets:DropdownAddButton(text, func, ...)
	local buttonRow
	if #buttonRows>0 then
		buttonRow = tremove(buttonRows)
		buttonRow:SetParent(dropFrame)
		buttonRow:Show()
		buttonRow.func = nil
		
		for k in ipairs(buttonRow.args) do
			buttonRow.args[k] = nil
		end
	else
		buttonRow = CreateFrame("BUTTON", nil, dropFrame)
		buttonRow.storage = buttonRows

		buttonRow.Text = buttonRow:CreateFontString(nil ,"OVERLAY","GameFontHighlightSmall")
		Recount2:AddFontString(buttonRow.Text)
		SetFontSize(buttonRow.Text, dropdownRowHeight)
		buttonRow.Text:SetPoint("LEFT", buttonRow, "LEFT", 1, 0)
		buttonRow.Text:SetJustifyH("LEFT")

		buttonRow.Highlight = buttonRow:CreateTexture(nil, "BACKGROUND")
		buttonRow.Highlight:SetTexture(1, 1, 0, 0.25)
		buttonRow.Highlight:Hide()
		buttonRow.Highlight:ClearAllPoints()
		buttonRow.Highlight:SetAllPoints(buttonRow)

		buttonRow:EnableMouse()
		buttonRow.OnClick = ButtonOnClick
		buttonRow.OnEnter = ButtonOnEnter
		buttonRow.OnLeave = ButtonOnLeave
		buttonRow:SetScript("OnClick", buttonRow.OnClick)
		buttonRow:SetScript("OnEnter", buttonRow.OnEnter)
		buttonRow:SetScript("OnLeave", buttonRow.OnLeave)

		buttonRow:SetHeight(dropdownRowHeight)

		buttonRow.args = Recount2:GetTable()
	end

	for i = 1, select("#", ...) do
		buttonRow.args[i] = select(i, ...)
	end

	buttonRow.func = func
	buttonRow.Text:SetText(text)
	local w = buttonRow.Text:GetStringWidth()

	if (w+4)>dropFrame:GetWidth() then
		dropFrame:SetWidth(w+4)
	end

	buttonRow:SetPoint("TOPLEFT", dropFrame, "TOPLEFT", 1, -position)
	buttonRow:SetPoint("TOPRIGHT", dropFrame, "TOPRIGHT", -1, -position)
	position = position + dropdownRowHeight + dropdownRowSpacing
	dropFrame:SetHeight(position)

	tinsert(inUse, buttonRow)
end
